/* Traverse all malloc'd blocks.
Copyright (C) 1996 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#define _MALLOC_INTERNAL
#include <libc-lock.h>
#include <malloc.h>

/* Locking.  */
__libc_lock_define (extern, __libc_malloc_lock)

__ptr_t
malloc_walk (__ptr_t posn)
{
  __malloc_size_t block, i;
  __ptr_t result = NULL;

  __libc_lock_lock (__libc_malloc_lock);

  if (posn)
    {
      /* Advance past this block.  */
      block = BLOCK (posn);
      if (_heapinfo[block].busy.type == 0)
	block += _heapinfo[block].busy.info.size;
      else
	block += 1 << _heapinfo[block].busy.type; /* XXX */
    }
  else
    /* Starting a new walk.  */
    block = 0;

  /* Search for BLOCK on the free list.  */
  if (block >= _heapindex)
    for (i = _heapindex; i < block; i = _heapinfo[i].free.next)
      {
	if (block < i + _heapinfo[i].free.size)
	  {
	    /* This block is free.  Skip it.  */
	    block += _heapinfo[block].free.size;
	    break;
	  }
      }
  else
    {
      i = _heapindex;
      do
	{
	  i = _heapinfo[i].free.prev;
	  if (i < block)
	    {
	      /* This block is free.  Skip it.  */
	      block += _heapinfo[block].free.size;
	      break;
	    }
	}
      while (block != _heapindex);
    }

  if (block == _heaplimit)
    /* Walked to the end of the heap.  */
    goto done;

  /* We have an allocated block.  What flavor is it?  */
  if (_heapinfo[block].busy.type == 0)
    /* Large block.  Return it.  */
    result = ADDRESS (block);
  else
    {
      /* A block of fragments.  */
      if (_heapinfo[block].busy.info.frag.first != 0)
	/* First fragment allocated.  Return it.  */
	result = ADDRESS (block);
      else
	{
	  /* Skip fragments until we find one not on the free list.  */
	  char *ptr = (char *) ADDRESS (block);
	  struct list *l;
	  do
	    {
	      ptr += _heapinfo[block].busy.type;
	      for (l = (struct list *) ADDRESS (block);
		   (__malloc_size_t) BLOCK (l) == block;
		   l = l->next)
		if (ptr == (char *) l)
		  break;
	    } while ((__malloc_size_t) BLOCK (l) == block);
	  result = ptr;
	}
    }

done:
  __libc_lock_unlock (__libc_malloc_lock);

  return result;
}
